import os


installation = {
    "loopfunctions": "/home/jkuckling/AutoMoDe-loopfunctions"
}

control_architectures = {
    "bt": {
        "installation_path": "/home/jkuckling/AutoMoDe-BT",
        "library_name": "libautomode_bt.so",
        "label":  "automode_controller_bt",
        "id": "automode_bt",
        "params": '<params bt-config="--nroot 3 --nchildroot 1 --n0 0 --nchild0 2 --n00 6 --c00 5 --p00 0.26 --n01 5 --a01 1 --p01 0"/>'
    },
    "fsm": {
        "installation_path": "/home/jkuckling/AutoMoDe-FSM",
        "library_name": "libautomode.so",
        "label":  "automode_controller",
        "id": "automode_fsm",
        "params": '<params fsm-config="--nstates 1 --s0 1"/>'
    },
    "nn": {
        "installation_path": "/home/jkuckling/EvoStick",
        "library_name": "controllers/libnn_rm_1dot1.so",
        "label":  "nn_rm_1dot1_controller",
        "id": "evostick",
        "params": '<params rab_time="10"/>"'
    }
}

visualization_techniques = {
    "": """<visualization>
    </visualization>""",
    "visu": """<visualization>
        <qt-opengl>
            <camera>
                <placement idx="0"
                           position="0,0,4.5"
                           look_at="0,0,0"
                           lens_focal_length="30" />
            </camera>
        </qt-opengl>
    </visualization>"""
}

simulation_actuators = """<actuators>
                <epuck_wheels implementation="default" noise_std_dev="0.05"/>
                <!--epuck_rgb_leds implementation="default" medium="leds"/-->
                <epuck_range_and_bearing implementation="medium" medium="rab" data_size="4" range="0.7"/>
            </actuators>"""

simulation_sensors = """<sensors>
                <epuck_proximity implementation="default" show_rays="false" noise_level="0.05" calibrated="true"/>
                <epuck_range_and_bearing implementation="medium" medium="rab" data_size="4" nois_std_deviation="1.5" loss_probability="0.85" calibrated="true"/>
                <epuck_light implementation="default" show_rays="false" noise_level="0.05" calibrated="true"/>
                <epuck_ground implementation="rot_z_only" noise_level="0.05" calibrated="true"/>
                <!--epuck_omnidirectional_camera implementation="rot_z_only" medium="leds" show_rays="false"/-->
            </sensors>"""

pseudoreality_actuators = """<actuators>
                <epuck_wheels implementation="default" noise_std_dev="0.15"/>
                <!--epuck_rgb_leds implementation="default" medium="leds"/-->
                <epuck_range_and_bearing implementation="medium" medium="rab" data_size="4" range="0.7"/>
            </actuators>"""

pseudoreality_sensors = """<sensors>
                <epuck_proximity implementation="default" show_rays="false" noise_level="0.05" calibrated="true"/>
                <epuck_range_and_bearing implementation="medium" medium="rab" data_size="4" loss_probability="0.90" calibrated="true"/>
                <epuck_light implementation="default" show_rays="false" noise_level="0.90" calibrated="true"/>
                <epuck_ground implementation="rot_z_only" noise_level="0.05" calibrated="true"/>
                <!--epuck_omnidirectional_camera implementation="rot_z_only" medium="leds" show_rays="false"/-->
            </sensors>"""


def save_template(file_path, file_base_name, template, architecture, visu):
    if visu:
        file_name = "{}/{}_{}_{}.argos".format(file_path, file_base_name.split(".")[0], architecture, visu)
    else:
        file_name = "{}/{}_{}.argos".format(file_path, file_base_name.split(".")[0], architecture)
    # print(template)
    with open(file_name, "w") as mission_file:
        mission_file.write(template)


def replace_controller(template, architecture):
    t = template.replace("CONTROL_PATH", control_architectures[architecture]["installation_path"])
    t = t.replace("CONTROL_LIBRARY", control_architectures[architecture]["library_name"])
    t = t.replace("CONTROL_PARAMS", control_architectures[architecture]["params"])
    t = t.replace("CONTROL_LABEL", control_architectures[architecture]["label"])
    t = t.replace("CONTROL_ID", control_architectures[architecture]["id"])
    return t


def replace_visu(template, visualization):
    t = template.replace("VISU", visualization_techniques[visualization])
    return t


def load_template(file_path, file_name):
    with open("templates/{}/{}".format(file_path, file_name)) as template_file:
        template = template_file.read()
    return template


def create_simulation_files(file_path, file_name):
    os.makedirs("simulation/{}".format(file_path), exist_ok=True)
    template = load_template(file_path, file_name)
    # run general replacements
    template = template.replace("LOOP_PATH", installation["loopfunctions"])
    template = template.replace("ACTUATORS", simulation_actuators)
    template = template.replace("SENSORS", simulation_sensors)
    # run replacements depending on control architecture
    for architecture in control_architectures:
        template_controller = replace_controller(template, architecture)
        for visu in visualization_techniques:
            template_visu = replace_visu(template_controller, visu)
            save_template("simulation/" + file_path, file_name, template_visu, architecture, visu)


def create_pseudoreality_files(file_path, file_name):
    os.makedirs("pseudoreality/{}".format(file_path), exist_ok=True)
    template = load_template(file_path, file_name)
    # run general replacements
    template = template.replace("LOOP_PATH", installation["loopfunctions"])
    template = template.replace("ACTUATORS", pseudoreality_actuators)
    template = template.replace("SENSORS", pseudoreality_sensors)
    # run replacements depending on control architecture
    for architecture in control_architectures:
        template_controller = replace_controller(template, architecture)
        for visu in visualization_techniques:
            template_visu = replace_visu(template_controller, visu)
            save_template("pseudoreality/" + file_path, file_name, template_visu, architecture, visu)


def create_missions_from_template(folder_hierarchy, template_file):
    file_path = "/".join(folder_hierarchy[1:])
    create_simulation_files(file_path, template_file)
    create_pseudoreality_files(file_path, template_file)


def process_templates_in_folder(folder_hierarchy):
    path = "/".join(folder_hierarchy)
    for item in os.listdir(path):
        if os.path.isdir("{}/{}".format(path, item)):
            new_hierarchy = folder_hierarchy + [item]
            process_templates_in_folder(new_hierarchy)
        else:
            print(path)
            create_missions_from_template(folder_hierarchy, item)


def process_templates():
    process_templates_in_folder(["templates"])


if __name__ == "__main__":
    process_templates()